#coding=utf8

# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.

'''
CloudControl addforward command
'''

from cccli.command import TqlCommand
from cccli.tunnel import Forward
from cccli.exception import *

class Command_addforward(TqlCommand):
    '''Create a forward'''

    def __init__(self, cli, argv0):
        TqlCommand.__init__(self, cli, argv0)
        self.tql_filter += "&con&r~'host|hv'"
        self.remove_option("--quiet")
        self.remove_option("--direct")
        self.add_option("-s", "--source", default="127.0.0.1",
                        help="Tunnel local source ip address and port. "
                        "Default 127.0.0.1")
        self.add_option("-d", "--destination", default="127.0.0.1:22",
                        help="Tunnel remote destination ip address and port. "
                        "Default 127.0.0.1:22")

    def __call__(self, argv):
        # argv check
        self.parse_args(argv)
        if len(self.args) != 1:
            raise cmdBadArgument()
        # checking source info
        splitted_source = map(str.strip, self.options.source.split(":"))
        self.source_ip = splitted_source[0]
        if len(splitted_source) == 1 or splitted_source[1] == "":
            self.source_port = 0
        else:
            self.source_port = int(splitted_source[1])
        # checking dest info
        splitted_dest = map(str.strip, self.options.destination.split(":"))
        self.dest_ip = splitted_dest[0]
        if len(splitted_dest) == 1 or splitted_dest[1] == "":
            self.dest_port = 0
        else:
            self.dest_port = int(splitted_dest[1])
        # list using method forward to check rights
        ans = self.rpccall("list", self.args[0], method="forward",
                           _direct=True, _status=False)
        for obj in ans["objects"]:
            # Create forward object
            fwd = Forward(self.rpc, obj["id"],
                          self.source_ip, self.source_port,
                          self.dest_ip, self.dest_port)
            # register it into cli
            self.cli.forwards[fwd.id] = fwd
            # start it
            fwd.start()
            # print
            self.cli.printer.out("Forward %s added." % fwd.id)

    def remote_functions(self):
        return set(("list", "forward"))
